#!/usr/bin/env python3
from kostal.DxsApi import DxsEntry


class ActualBattery:
    VOLTAGE = 33556226
    CHARGE = 33556229
    CURRENT = 33556238
    CURRENT_DIR = 33556230
    CHARGE_CYCLES = 33556228
    TEMPERATURE = 33556227

    def __init__(self, inverter) -> None:
        self.__inverter = inverter

    async def battery_voltage(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.VOLTAGE)) \
            .get_entry_by_id(self.VOLTAGE)

    async def battery_charge(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.CHARGE)) \
            .get_entry_by_id(self.CHARGE)

    async def battery_current(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.CURRENT)) \
            .get_entry_by_id(self.CURRENT)

    async def battery_current_dir(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.CURRENT_DIR)) \
            .get_entry_by_id(self.CURRENT_DIR)

    async def battery_charge_cycles(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.CHARGE_CYCLES)) \
            .get_entry_by_id(self.CHARGE_CYCLES)

    async def battery_temp(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.TEMPERATURE)) \
            .get_entry_by_id(self.TEMPERATURE)



#!/usr/bin/env python3
from typing import List
import json


class DxsEntry:
    def __init__(self, dxsId, value):
        super().__init__()
        self.dxsId = dxsId
        self.value = value

    @classmethod
    def from_json(cls, data):
        return cls(**data)


class DxsSessionData:
    def __init__(self, sessionId, roleId):
        super().__init__()
        self.session_id = sessionId
        self.role_id = roleId

    @classmethod
    def from_json(cls, data):
        return cls(**data)


class DxsStatus:
    def __init__(self, code):
        super().__init__()
        self.code = code

    @classmethod
    def from_json(cls, data):
        return cls(**data)


class DxsResponse:
    __dxs_entries: List[DxsEntry]
    session: DxsSessionData
    status: DxsStatus

    def __init__(self, dxsEntries, session, status):
        super().__init__()
        self.__dxs_entries = list(map(DxsEntry.from_json, dxsEntries))
        self.__session = DxsSessionData.from_json(session)
        self.status = DxsStatus.from_json(status)

    def get_entry_by_id(self, dxs_id: int):
        for i in self.__dxs_entries:
            if i.dxsId == dxs_id:
                return i
        return None

    @classmethod
    def from_json(cls, json_str):
        json_dict = json.loads(json_str)
        return cls(**json_dict)



#!/usr/bin/env python3
from kostal.DxsApi import DxsEntry


class ActualGenerator:
    DC_1_VOLTAGE = 33555202
    DC_1_CURRENT = 33555201
    DC_1_POWER = 33555203
    DC_2_VOLTAGE = 33555458
    DC_2_CURRENT = 33555457
    DC_2_POWER = 33555459
    DC_3_VOLTAGE = 33555714
    DC_3_CURRENT = 33555713
    DC_3_POWER = 33555715

    def __init__(self, inverter) -> None:
        self.__inverter = inverter

    async def generator_dc_1_voltage(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.DC_1_VOLTAGE)) \
            .get_entry_by_id(self.DC_1_VOLTAGE)

    async def generator_dc_1_current(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.DC_1_CURRENT)) \
            .get_entry_by_id(self.DC_1_CURRENT)

    async def generator_dc_1_power(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.DC_1_POWER)) \
            .get_entry_by_id(self.DC_1_POWER)

    async def generator_dc_2_voltage(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.DC_2_VOLTAGE)) \
            .get_entry_by_id(self.DC_2_VOLTAGE)

    async def generator_dc_2_current(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.DC_2_CURRENT)) \
            .get_entry_by_id(self.DC_2_CURRENT)

    async def generator_dc_2_power(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.DC_2_POWER)) \
            .get_entry_by_id(self.generator_)

    async def generator_dc_3_voltage(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.DC_3_VOLTAGE)) \
            .get_entry_by_id(self.DC_3_VOLTAGE)

    async def generator_dc_3_current(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.DC_3_CURRENT)) \
            .get_entry_by_id(self.DC_3_CURRENT)

    async def generator_dc_3_power(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.DC_3_POWER)) \
            .get_entry_by_id(self.DC_3_POWER)



#!/usr/bin/env python3
from kostal.DxsApi import DxsEntry


class SettingsGeneral:
    INVERTER_NAME = 16777984
    INVERTER_MAKE = 16780544

    def __init__(self, inverter) -> None:
        self.__inverter = inverter

    async def inverter_name(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.INVERTER_NAME)) \
            .get_entry_by_id(self.INVERTER_NAME)

    async def inverter_make(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.INVERTER_MAKE)) \
            .get_entry_by_id(self.INVERTER_MAKE)



#!/usr/bin/env python3
from kostal.DxsApi import DxsEntry


class ActualSZeroIn:
    S0_IN_PULSE_COUNT = 184549632
    LOG_INTERVAL = 150995968

    def __init__(self, inverter) -> None:
        self.__inverter = inverter

    async def s0_in_pulse_count(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.S0_IN_PULSE_COUNT)) \
            .get_entry_by_id(self.S0_IN_PULSE_COUNT)

    async def s0_in_log_interval(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.LOG_INTERVAL)) \
            .get_entry_by_id(self.LOG_INTERVAL)



#!/usr/bin/env python3
import aiohttp
import async_timeout
import asyncio
import json
from typing import List

from kostal.DxsApi import DxsResponse
from kostal.ActualAnalogInputs import ActualAnalogInputs
from kostal.ActualBattery import ActualBattery
from kostal.ActualGrid import ActualGrid
from kostal.ActualHome import ActualHome
from kostal.ActualGenerator import ActualGenerator
from kostal.ActualGrid import ActualGrid
from kostal.ActualHome import ActualHome
from kostal.ActualSZeroIn import ActualSZeroIn
from kostal.Home import Home
from kostal.InfoVersions import InfoVersions
from kostal.SettingsGeneral import SettingsGeneral
from kostal.StatisticDay import StatisticDay
from kostal.StatisticTotal import StatisticTotal

DXS_ENDPOINT = "/api/dxs.json"
LOG_DATA_ENDPOINT = "/LogDaten.dat"

DEFAULT_USERNAME = 'pvserver'
DEFAULT_PASSWORD = 'pvwr'


class Piko:
    """ 
    Interface to communicate with the Kostal Piko over http request
    Attributes:
        session     The AIO session
        url         the url for reaching of the inverter
                    (i.e. http://192.168.0.123)
        username    inverter username (default pvserver)
        password    inverter password (default pvwr)
        timeout     HTTP timeout in seconds
     """

    def __init__(self, session: aiohttp.ClientSession, url: str = None, username: str = DEFAULT_USERNAME,
                 password: str = DEFAULT_PASSWORD,
                 timeout: int = 10):
        """ Constructor """
        if url is None:
            raise ValueError("Parameter url can not be None")

        self.__client = session
        self.__url = url + DXS_ENDPOINT
        self.__username = username
        self.__password = password
        self.__timeout = timeout

        self.actualAnalogInputs = ActualAnalogInputs(self)
        self.actualBattery = ActualBattery(self)
        self.actualGenerator = ActualGenerator(self)
        self.actualGrid = ActualGrid(self)
        self.actualHome = ActualHome(self)
        self.actualSZeroIn = ActualSZeroIn(self)
        self.home = Home(self)
        self.infoVersions = InfoVersions(self)
        self.settingsGeneral = SettingsGeneral(self)
        self.statisticDay = StatisticDay(self)
        self.statisticTotal = StatisticTotal(self)

    async def __fetch_data(self, request_params):
        """ fetch data """
        auth = aiohttp.BasicAuth(self.__username, self.__password)
        try:
            async with async_timeout.timeout(self.__timeout):
                async with self.__client.get(self.__url,
                                             params=request_params,
                                             auth=auth,
                                             timeout=self.__timeout) as resp:
                    assert resp.status == 200
                    return await resp.json(content_type='text/plain')
        except (asyncio.TimeoutError, aiohttp.ClientError):
            raise ConnectionError(
                "Connection to Kostal device failed at {}.".format(self.__url))
        except json.JSONDecodeError:
            raise ValueError(
                "Device returned a non-JSON reply at {}.".format(self.__url))

    async def fetch_props(self, *prop_ids):
        response = await self.__fetch_data({'dxsEntries': prop_ids})
        return DxsResponse(**response)



#!/usr/bin/env python3
from kostal.DxsApi import DxsEntry


class InfoVersions:
    VERSION_UI = 16779267
    VERSION_FW = 16779265
    VERSION_HW = 16779266
    VERSION_PAR = 16779268
    SERIAL_NUMBER = 16777728
    ARTICLE_NUMBER = 16777472
    COUNTRY_SETTINGS_NAME = 16779522
    COUNTRY_SETTINGS_VERSION = 16779521

    def __init__(self, inverter) -> None:
        self.__inverter = inverter

    async def version_ui(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.VERSION_UI)) \
            .get_entry_by_id(self.VERSION_UI)

    async def version_fw(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.VERSION_FW)) \
            .get_entry_by_id(self.VERSION_FW)

    async def version_hw(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.VERSION_HW)) \
            .get_entry_by_id(self.VERSION_HW)

    async def version_par(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.VERSION_PAR)) \
            .get_entry_by_id(self.VERSION_PAR)

    async def serial_nr(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.SERIAL_NUMBER)) \
            .get_entry_by_id(self.SERIAL_NUMBER)

    async def article_nr(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.ARTICLE_NUMBER)) \
            .get_entry_by_id(self.ARTICLE_NUMBER)

    async def country_settings_name(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.COUNTRY_SETTINGS_NAME)) \
            .get_entry_by_id(self.COUNTRY_SETTINGS_NAME)

    async def country_settings_version(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.COUNTRY_SETTINGS_VERSION)) \
            .get_entry_by_id(self.COUNTRY_SETTINGS_VERSION)



#!/usr/bin/env python3
from kostal.DxsApi import DxsEntry


class ActualGrid:
    GRID_OUTPUT_POWER = 67109120
    GRID_FREQ = 67110400
    GRID_COS_PHI = 67110656
    GRID_LIMITATION = 67110144
    GRID_VOLTAGE_L1 = 67109378
    GRID_CURRENT_L1 = 67109377
    GRID_POWER_L1 = 67109379
    GRID_VOLTAGE_L2 = 67109634
    GRID_CURRENT_L2 = 67109633
    GRID_POWER_L2 = 67109635
    GRID_VOLTAGE_L3 = 67109890
    GRID_CURRENT_L3 = 67109889
    GRID_POWER_L3 = 67109891

    def __init__(self, inverter) -> None:
        self.__inverter = inverter

    async def grid_output_power(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.GRID_OUTPUT_POWER)) \
            .get_entry_by_id(self.GRID_OUTPUT_POWER)

    async def grid_frequency(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.GRID_FREQ)) \
            .get_entry_by_id(self.GRID_FREQ)

    async def grid_cos_phi(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.GRID_COS_PHI)) \
            .get_entry_by_id(self.GRID_COS_PHI)

    async def grid_limitation(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.GRID_LIMITATION)) \
            .get_entry_by_id(self.GRID_LIMITATION)

    async def grid_voltage_l1(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.GRID_VOLTAGE_L1)) \
            .get_entry_by_id(self.GRID_VOLTAGE_L1)

    async def grid_current_l1(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.GRID_CURRENT_L1)) \
            .get_entry_by_id(self.GRID_CURRENT_L1)

    async def grid_power_l1(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.GRID_POWER_L1)) \
            .get_entry_by_id(self.GRID_POWER_L1)

    async def grid_voltage_l2(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.GRID_VOLTAGE_L2)) \
            .get_entry_by_id(self.GRID_VOLTAGE_L2)

    async def grid_current_l2(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.GRID_CURRENT_L2)) \
            .get_entry_by_id(self.GRID_CURRENT_L2)

    async def grid_power_l2(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.GRID_POWER_L2)) \
            .get_entry_by_id(self.GRID_POWER_L2)

    async def grid_voltage_l3(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.GRID_VOLTAGE_L3)) \
            .get_entry_by_id(self.GRID_VOLTAGE_L3)

    async def grid_current_l3(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.GRID_CURRENT_L3)) \
            .get_entry_by_id(self.GRID_CURRENT_L3)

    async def grid_power_l3(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.GRID_POWER_L3)) \
            .get_entry_by_id(self.GRID_POWER_L3)



#!/usr/bin/env python3
from kostal.DxsApi import DxsEntry


class StatisticTotal:
    YIELD = 251658753
    OPERATING_TIME = 251658496
    HOME_CONSUMPTION = 251659009
    SELF_CONSUMPTION = 251659265
    SELF_CONSUMPTION_RATE = 251659280
    AUTONOMY_DEGREE = 251659281

    def __init__(self, inverter) -> None:
        self.__inverter = inverter

    async def total_yield(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.YIELD)) \
            .get_entry_by_id(self.YIELD)

    async def total_home_consumption(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.HOME_CONSUMPTION)) \
            .get_entry_by_id(self.HOME_CONSUMPTION)

    async def total_self_consumption(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.SELF_CONSUMPTION)) \
            .get_entry_by_id(self.SELF_CONSUMPTION)

    async def total_self_consumption_rate(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.SELF_CONSUMPTION_RATE)) \
            .get_entry_by_id(self.SELF_CONSUMPTION_RATE)

    async def total_autonomy_degree(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.AUTONOMY_DEGREE)) \
            .get_entry_by_id(self.AUTONOMY_DEGREE)

    async def total_operating_time(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.OPERATING_TIME)) \
            .get_entry_by_id(self.OPERATING_TIME)



#!/usr/bin/env python3
from kostal.DxsApi import DxsEntry


class Home:
    DC_POWER_PV = 33556736
    AC_POWER = 67109120
    OWN_CONSUMPTION = 83888128
    BATTERY_STATE_OF_CHARGE = 33556229
    OPERATING_STATUS = 16780032
    # TODO: statusCodes

    def __init__(self, inverter) -> None:
        self.__inverter = inverter

    async def home_dc_power_pv(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.DC_POWER_PV)) \
            .get_entry_by_id(self.DC_POWER_PV)

    async def home_ac_power(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.AC_POWER)) \
            .get_entry_by_id(self.AC_POWER)

    async def home_own_consumption(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.OWN_CONSUMPTION)) \
            .get_entry_by_id(self.OWN_CONSUMPTION)

    async def home_battery_state_of_charge(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.BATTERY_STATE_OF_CHARGE)) \
            .get_entry_by_id(self.BATTERY_STATE_OF_CHARGE)

    async def inverter_operating_status(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.OPERATING_STATUS)) \
            .get_entry_by_id(self.OPERATING_STATUS)



#!/usr/bin/env python3
from kostal.DxsApi import DxsEntry


class ActualHome:
    ACT_HOME_CONSUMPTION_SOLAR = 83886336
    ACT_HOME_CONSUMPTION_BATTERY = 83886592
    ACT_HOME_CONSUMPTION_GRID = 83886848
    PHASE_SELECTIVE_CONSUMPTION_L1 = 83887106
    PHASE_SELECTIVE_CONSUMPTION_L2 = 83887362
    PHASE_SELECTIVE_CONSUMPTION_L3 = 83887618

    def __init__(self, inverter) -> None:
        self.__inverter = inverter

    async def home_consumption_solar(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.ACT_HOME_CONSUMPTION_SOLAR)) \
            .get_entry_by_id(self.ACT_HOME_CONSUMPTION_SOLAR)

    async def home_consumption_battery(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.ACT_HOME_CONSUMPTION_BATTERY)) \
            .get_entry_by_id(self.ACT_HOME_CONSUMPTION_BATTERY)

    async def home_consumption_grid(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.ACT_HOME_CONSUMPTION_GRID)) \
            .get_entry_by_id(self.ACT_HOME_CONSUMPTION_GRID)

    async def home_phase_consumption_l1(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.PHASE_SELECTIVE_CONSUMPTION_L1)) \
            .get_entry_by_id(self.PHASE_SELECTIVE_CONSUMPTION_L1)

    async def home_phase_consumption_l2(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.PHASE_SELECTIVE_CONSUMPTION_L2)) \
            .get_entry_by_id(self.PHASE_SELECTIVE_CONSUMPTION_L2)

    async def home_phase_consumption_l3(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.PHASE_SELECTIVE_CONSUMPTION_L3)) \
            .get_entry_by_id(self.PHASE_SELECTIVE_CONSUMPTION_L3)



#!/usr/bin/env python3


from setuptools import setup, find_packages

# Get the long description from the README file
with open('README.md', 'r') as fh:
    long_description = fh.read()

setup(
    name='pykostal',
    version='0.0.2',
    author='Andreas Rehn',
    author_email='rehn.andreas86@gmail.com',
    url='https://github.com/DAMEK86/pykostal',
    description='package to communicate with Kostal Piko invertes',
    long_description=long_description,
    long_description_content_type='text/markdown',
    py_modules=['kostal'],
    packages=find_packages(),
    license='MIT',
    classifiers=[
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3',
    install_requires=[
        'async_timeout',
        'aiohttp'
    ],
    extras_require={
        "dev": [
            "pytest>=3.7",
            "twine",
            "tox",
        ],
    },
)



#!/usr/bin/env python3
from kostal.DxsApi import DxsEntry


class ActualAnalogInputs:
    ANALOG1 = 167772417
    ANALOG2 = 167772673
    ANALOG3 = 167772929
    ANALOG4 = 167773185

    def __init__(self, inverter) -> None:
        self.__inverter = inverter

    async def analog_1(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.ANALOG1)) \
            .get_entry_by_id(self.ANALOG1)

    async def analog_2(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.ANALOG2)) \
            .get_entry_by_id(self.ANALOG2)

    async def analog_3(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.ANALOG3)) \
            .get_entry_by_id(self.ANALOG3)

    async def analog_4(self) -> DxsEntry:
        return (await self.__inverter.fetch_props(self.ANALOG4)) \
            .get_entry_by_id(self.ANALOG4)
